-- AddOn by Daisey, Argent Dawn, Europe. Please give me credit when you want to redistribute or modify this addon! Contact: coenvdwel@planet.nl
-- Coen van der Wel, Eindhoven, the Netherlands.

iLoot_Version = "1.4";


--------------------------------------------------------------------------------------------------------------[ FUNCTIONS ]----------------------

-- display a message to the user in the chat pane
function iLoot_Message(msg)

	-- prevent nil messages
	if(not msg) then return; end;
	
	-- initialise vars
	local pos, indent, prefix = 0, "", iLoot_Loc["Prefix"];
	for i=0, iLoot_Loc["Prefix"]:len() do indent = indent.." "; end;
	
	-- process each newline individually
	for st, sp in function() return string.find(msg, "\n", pos, true) end do
	
		-- arrange prefix accordingly
		if(pos > 0) then prefix = indent; end;
		
		-- output message
		DEFAULT_CHAT_FRAME:AddMessage(prefix..string.sub(msg, pos, st-1), iLoot_Red, iLoot_Green, iLoot_Blue);
		
		-- repeat by incrementing cursor
		pos = sp + 1;
		
	end; -- for st, sp
	
	-- and let's not forget to print the last bit, too
	DEFAULT_CHAT_FRAME:AddMessage(prefix..string.sub(msg, pos), iLoot_Red, iLoot_Green, iLoot_Blue);
	
end; -- function iLoot_Message

-- update color vars
function iLoot_UpdateColor()

	-- get the colorframe selected color
	iLoot_Red, iLoot_Green, iLoot_Blue = ColorPickerFrame:GetColorRGB();
	
	-- save to saved variables
	iLoot_RedSAVE, iLoot_GreenSAVE, iLoot_BlueSAVE = iLoot_Red, iLoot_Green, iLoot_Blue;
	
end; -- function iLoot_UpdateColor

-- log a looting player
function iLoot_LogLoot(player, boss)

	-- create table value if it doesn't exist, yet
	if(not iLoot_Log[player]) then
		iLoot_Log[player] = {};
		table.insert(iLoot_Log[player], { 0 });
	end;
	
	-- add to log
	table.insert(iLoot_Log[player], { Date = date("%m/%d/%y %H:%M:%S"), Boss = boss});
	iLoot_Log[player][1][1] = iLoot_Log[player][1][1] + 1;
	
end; -- function iLoot_LogLoot

-- return if anyone in the raid is dead or not
function iLoot_AnyoneIsDead()

	for i = 1, GetNumRaidMembers() do
		if(UnitIsDeadOrGhost("raid"..i)) then return true; end;
	end; -- for i
	
	return false;

end; -- function iLoot_AnyoneIsDead

-- process slash command input
function iLoot_SlashCommandHandler(cmd)

	-- process parameters
	local pos, cmds = 0, {};
	for st, sp in function() return string.find(cmd, " ", pos, true) end do
		table.insert(cmds, string.lower(string.sub(cmd, pos, st-1))); pos = sp + 1;
	end; table.insert(cmds, string.lower(string.sub(cmd, pos)));
	
	-- change text color
	if(cmds[1] == "color") then
		ColorPickerFrame:Hide();
		ColorPickerFrame.func = iLoot_UpdateColor;
		ColorPickerFrame:SetColorRGB(iLoot_Red, iLoot_Green, iLoot_Blue);
		ColorPickerFrame.hasOpacity = false; ColorPickerFrame.opacityFunc = nil;
		ColorPickerFrame:Show();
		
	-- enable
	elseif(cmds[1] == "enable") then
		iLoot_Enabled = true;
		iLoot_EnabledSAVE = true;
		iLoot_Message(iLoot_Loc["Enabled"]);
		
	-- disable
	elseif(cmds[1] == "disable") then
		iLoot_Enabled = false;
		iLoot_EnabledSAVE = false;
		iLoot_Message(iLoot_Loc["Disabled"]);
		
	-- toggle filter
	elseif(cmds[1] == "filter") then
		-- check if the command was valid
		if((cmds[2] ~= nil) and ((cmds[2] == "on") or (cmds[2] == "off"))) then
			iLoot_Filter = (cmds[2] == "on");
			iLoot_FilterSAVE = iLoot_Filter;
			iLoot_Message(iLoot_Loc["Filter "..cmds[2]]);
		-- report if it wasn't
		else iLoot_Message(iLoot_Loc["Invalid"].." "..iLoot_Loc["Help"]); end;
	
	-- clear log
	elseif(cmds[1] == "clearlog") then
		iLoot_Log = {};
		iLoot_Message(iLoot_Loc["Cleared log"]);
		
	-- show version
	elseif(cmds[1] == "version") then
		iLoot_Message("v"..iLoot_Version);
	
	-- report error/usage message
	elseif(cmd:len() > 0) then iLoot_Message(iLoot_Loc["Invalid"].." "..iLoot_Loc["Help"]);
	else iLoot_Message(iLoot_Loc["Help"]); end;
	
end; -- function iLoot_SlashCommandHandler


---------------------------------------------------------------------------------------------------------------[ SETUP ]-------------------------

-- declare vars
iLoot_Enabled = false;
iLoot_Red, iLoot_Green, iLoot_Blue = 0.95, 0.85, 0.15;

-- hook into AceEvent-2.0 lib
iLoot_Lib = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0");

-- assign slashcommand handler
SLASH_ILOOT1, SLASH_ILOOT2, SLASH_ILOOT3 = "/iloot", "/iLoot", "/ILoot";
SlashCmdList["ILOOT"] = function(cmd)
	iLoot_SlashCommandHandler(cmd);
end;


-----------------------------------------------------------------------------------------------------------------[ XML ]-------------------------

-- none


-----------------------------------------------------------------------------------------------------------------[ EVENTS ]----------------------

-- construct listener
function iLoot_Lib:OnEnable()

	-- load all saved vars if present, or if not, set them
	if(iLoot_RedSAVE ~= nil) then iLoot_Red = iLoot_RedSAVE; else iLoot_RedSAVE = iLoot_Red; end;
	if(iLoot_GreenSAVE ~= nil) then iLoot_Green = iLoot_GreenSAVE; else iLoot_GreenSAVE = iLoot_Green; end;
	if(iLoot_BlueSAVE ~= nil) then iLoot_Blue = iLoot_BlueSAVE; else iLoot_BlueSAVE = iLoot_Blue; end;
	if(iLoot_FilterSAVE ~= nil) then iLoot_Filter = iLoot_FilterSAVE; else iLoot_FilterSAVE = iLoot_Filter; end;
	if(iLoot_Log == nil) then iLoot_Log = {}; end;
	
	-- load the enabled var, or start enabled
	if(iLoot_EnabledSAVE ~= nil) then iLoot_Enabled = iLoot_EnabledSAVE; else
		iLoot_EnabledSAVE = true; iLoot_Enabled = true;
	end;
	
	-- start registering events
	self:RegisterEvent("CHAT_MSG_ADDON");
	
end; -- function iLoot_Lib:OnEnable

-- deconstruct listener
function iLoot_Lib:OnDisable()

	-- stop registering events
	self:UnregisterAllEvents();
	
end; -- function iLoot_Lib:OnDisable

-- define local vars
local name, bosstype, mlooter; -- storage data
local looterCheck, bossCheck, filterCheck, chestCheck; -- boolean data
local lastLootedBoss = ""; -- register data

-- make sure we catch events and process them
function iLoot_Lib:CHAT_MSG_ADDON(pre, event, channel, player)
	
	-- if we're enabled
	if(iLoot_Enabled) then
	
		-- if it's the loot event we want
		if(((event == "LOOT_OPENED") or (pre == "LOOT_OPENED")) and (player)) then
		
			-- get info
			name = UnitName(player.."-target");
			bosstype = UnitClassification(player.."-target");
			loottype = UnitIsPlayer(player.."-target");
			_, _, mlooter = GetLootMethod();
			
			-- set checks
			looterCheck = (mlooter and (UnitName("raid"..mlooter) ~= player) or (not mlooter)) and (UnitName("player") ~= player);
			looterCheck = looterCheck and UnitInRange(player);
			bossCheck = name and (bosstype == "worldboss") and (not loottype);
			filterCheck = (not iLoot_Filter) or (lastLootedBoss ~= name);
			chestCheck = not name;
			
			-- check
			if(looterCheck and bossCheck and filterCheck) then
				
				-- report to own chat
				iLoot_Message(player.." looted "..name.."!");
				
				-- log this loot
				iLoot_LogLoot(player, name);
				
				-- register boss loot
				lastLootedBoss = name;
				
			-- else, if we're looting while someone is dead
			elseif(iLoot_AnyoneIsDead() and (not chestCheck)) then
				
				-- report to own chat
				iLoot_Message(player.." looted "..name.." while someone was dead!");

				-- log this loot
				iLoot_LogLoot(player, name.." while someone was dead!");
				
			-- else, if we're possibly looting a treasure chest
			elseif(looterCheck and chestCheck) then
			
				-- report to own chat
				iLoot_Message(player.." possibly looted a treasure chest!");
				
				-- log this loot
				iLoot_LogLoot(player, "A treasure chest");
				
			end; -- if(looterCheck and bossCheck and filterCheck), elseif(looterCheck and chestCheck), elseif(iLoot_AnyoneIsDead())
			
		end; -- if(((event == "LOOT_OPENED") or (pre == "LOOT_OPENED")) and (player))
	
	end; -- if(iLoot_Enabled)
	
end; -- function iLoot_Lib:CHAT_MSG_ADDON